@@ -1,7 +1,7 @@
 // Copyright (c) Jupyter Development Team.
 // Distributed under the terms of the Modified BSD License.
 import { expect, galata, test } from '@jupyterlab/galata';
-import { ElementHandle } from '@playwright/test';
+import { Locator } from '@playwright/test';
 import * as path from 'path';
 
 const fileName = 'windowed_notebook.ipynb';
@@ -28,26 +28,16 @@ test.beforeEach(async ({ page, tmpPath }) => {
   );
 });
 
-async function getInnerHeight(panel: ElementHandle<Element>) {
+async function getInnerHeight(panel: Locator) {
   return parseInt(
-    await panel.$eval(
-      '.jp-WindowedPanel-inner',
-      node => (node as HTMLElement).style.height
-    ),
-    10
-  );
-}
-async function getWindowHeight(panel: ElementHandle<Element>) {
-  return parseInt(
-    await panel.$eval(
-      '.jp-WindowedPanel-viewport',
-      node => (node as HTMLElement).style.minHeight
-    ),
+    await panel
+      .locator('.jp-WindowedPanel-inner')
+      .evaluate(node => (node as HTMLElement).style.height),
     10
   );
 }
 
-test('should update window height on resize', async ({ page, tmpPath }) => {
+test('should update displayed cells on resize', async ({ page, tmpPath }) => {
   // Note: this needs many small cells so that they get added during resize changing height.
   const notebookName = '20_empty_cells.ipynb';
   await page.contents.uploadFile(
@@ -56,20 +46,21 @@ test('should update window height on resize', async ({ page, tmpPath }) => {
   );
   await page.notebook.openByPath(`${tmpPath}/${notebookName}`);
 
-  const notebook = await page.notebook.getNotebookInPanel();
+  const notebook = await page.notebook.getNotebookInPanelLocator();
 
-  // Measure height when the notebook is open but launcher closed
-  const fullHeight = await getWindowHeight(notebook);
+  const cell = notebook.locator('.jp-Cell[data-windowed-list-index="10"]');
+
+  // Cell should be visible
+  await expect.soft(cell).toBeVisible();
 
   // Add a new launcher below the notebook
   await page.evaluate(async () => {
     const widget = await window.jupyterapp.commands.execute('launcher:create');
     window.jupyterapp.shell.add(widget, 'main', { mode: 'split-bottom' });
   });
-  // Measure height after splitting the dock area
-  const heightAfterSplit = await getWindowHeight(notebook);
 
-  expect(heightAfterSplit).toBeLessThan(fullHeight);
+  // The cell should no longer be visible
+  await expect.soft(cell).not.toBeVisible();
 
   // Resize the dock panel, increasing the notebook height/decreasing the launcher height.
   const resizeHandle = page.locator(
@@ -77,27 +68,31 @@ test('should update window height on resize', async ({ page, tmpPath }) => {
   );
   await resizeHandle.dragTo(page.locator('#jp-main-statusbar'));
 
-  // Measure height after resizing
-  const heightAfterResize = await getWindowHeight(notebook);
-
-  expect(heightAfterResize).toBeGreaterThan(heightAfterSplit);
+  // The cell should be visible aqain
+  await expect.soft(cell).toBeVisible();
 });
 
 test('should not update height when hiding', async ({ page, tmpPath }) => {
   await page.notebook.openByPath(`${tmpPath}/${fileName}`);
+  const notebook = await page.notebook.getNotebookInPanelLocator();
+  let initialHeight = 0;
+  let previousHeight = 0;
+  let counter = 0;
 
   // Wait to ensure the rendering logic is stable.
-  await page.waitForTimeout(200);
+  do {
+    previousHeight = initialHeight;
+    await page.waitForTimeout(300);
 
-  const notebook = await page.notebook.getNotebookInPanel();
-  const initialHeight = await getInnerHeight(notebook);
+    initialHeight = await getInnerHeight(notebook!);
+  } while (previousHeight !== initialHeight && counter++ < 10);
 
-  expect(initialHeight).toBeGreaterThan(0);
+  expect.soft(initialHeight).toBeGreaterThan(0);
 
   // Add a new launcher covering the notebook.
   await page.menu.clickMenuItem('File>New Launcher');
 
-  const innerHeight = await getInnerHeight(notebook);
+  const innerHeight = await getInnerHeight(notebook!);
 
   expect(innerHeight).toEqual(initialHeight);
 });
@@ -111,14 +106,14 @@ test('should hide first inactive code cell when scrolling down', async ({
   // Activate >second< cell
   await page.notebook.selectCells(1);
   // Test if the >first< (now inactive) cell gets detached
-  const h = await page.notebook.getNotebookInPanel();
-  const firstCellSelector = '.jp-Cell[data-windowed-list-index="0"]';
-  const firstCell = await h!.waitForSelector(firstCellSelector);
+  const h = await page.notebook.getNotebookInPanelLocator();
+  const firstCell = h!.locator('.jp-Cell[data-windowed-list-index="0"]');
+  await firstCell.waitFor();
 
   const bbox = await h!.boundingBox();
   await page.mouse.move(bbox!.x, bbox!.y);
   await Promise.all([
-    firstCell.waitForElementState('hidden'),
+    firstCell.waitFor({ state: 'hidden' }),
     page.mouse.wheel(0, 600)
   ]);
 
@@ -135,25 +130,25 @@ test('should reattached inactive code cell when scrolling back into the viewport
   // Activate >second< cell
   await page.notebook.selectCells(1);
   // Test if the >first< (now inactive) cell gets re-attached
-  const h = await page.notebook.getNotebookInPanel();
-  const firstCellSelector = '.jp-Cell[data-windowed-list-index="0"]';
-  const firstCell = await h!.waitForSelector(firstCellSelector);
+  const h = await page.notebook.getNotebookInPanelLocator();
+  const firstCell = h!.locator('.jp-Cell[data-windowed-list-index="0"]');
+  await firstCell.waitFor();
 
   const bbox = await h!.boundingBox();
   await page.mouse.move(bbox!.x, bbox!.y);
   await Promise.all([
-    firstCell.waitForElementState('hidden'),
-    h!.waitForSelector('.jp-MarkdownCell[data-windowed-list-index="6"]'),
+    firstCell.waitFor({ state: 'hidden' }),
+    h!.locator('.jp-MarkdownCell[data-windowed-list-index="6"]').waitFor(),
     page.mouse.wheel(0, 1200)
   ]);
 
   await Promise.all([
-    firstCell.waitForElementState('visible'),
+    firstCell.waitFor({ state: 'visible' }),
     page.mouse.wheel(0, -1200)
   ]);
 
   // Check that the input area is back
-  expect(await firstCell.waitForSelector('.jp-InputArea')).toBeDefined();
+  await expect(firstCell.locator('.jp-InputArea')).toHaveCount(1);
 });
 
 test('should not detach active code cell input when scrolling down', async ({
@@ -163,19 +158,19 @@ test('should not detach active code cell input when scrolling down', async ({
   await page.notebook.openByPath(`${tmpPath}/${fileName}`);
 
   await page.notebook.selectCells(0);
-  const h = await page.notebook.getNotebookInPanel();
-  const firstCellSelector = '.jp-Cell[data-windowed-list-index="0"]';
-  const firstCell = await h!.waitForSelector(firstCellSelector);
+  const h = await page.notebook.getNotebookInPanelLocator();
+  const firstCell = h!.locator('.jp-Cell[data-windowed-list-index="0"]');
+  await firstCell.waitFor();
 
   const bbox = await h!.boundingBox();
   await page.mouse.move(bbox!.x, bbox!.y);
   await Promise.all([
-    firstCell.waitForElementState('hidden'),
+    firstCell.waitFor({ state: 'hidden' }),
     page.mouse.wheel(0, 1200)
   ]);
 
   // Check the input is still defined
-  expect(await firstCell.waitForSelector('.jp-InputArea')).toBeDefined();
+  await expect(firstCell.locator('.jp-InputArea')).toHaveCount(1);
 });
 
 for (const cellType of ['code', 'markdown']) {
@@ -187,14 +182,14 @@ for (const cellType of ['code', 'markdown']) {
 
     await page.notebook.setCellType(0, cellType);
     await page.notebook.enterCellEditingMode(0);
-    const h = await page.notebook.getNotebookInPanel();
-    const firstCellSelector = '.jp-Cell[data-windowed-list-index="0"]';
-    const firstCell = await h!.waitForSelector(firstCellSelector);
+    const h = await page.notebook.getNotebookInPanelLocator();
+    const firstCell = h!.locator('.jp-Cell[data-windowed-list-index="0"]');
+    await firstCell.waitFor();
 
     const bbox = await h!.boundingBox();
     await page.mouse.move(bbox!.x, bbox!.y);
     await Promise.all([
-      firstCell.waitForElementState('hidden'),
+      firstCell.waitFor({ state: 'hidden' }),
       page.mouse.wheel(0, 1200)
     ]);
 
@@ -202,43 +197,109 @@ for (const cellType of ['code', 'markdown']) {
     await page.keyboard.type('TEST', { delay: 150 });
 
     // Expect the cell to become visible again
-    await firstCell.waitForElementState('visible');
+    await firstCell.waitFor({ state: 'visible' });
 
     // Expect the text to populate the cell editor
-    const firstCellInput = await page.notebook.getCellInput(0);
-    expect(await firstCellInput.textContent()).toContain('TEST');
+    const firstCellInput = await page.notebook.getCellInputLocator(0);
+    expect(await firstCellInput!.textContent()).toContain('TEST');
   });
 }
 
-test('should scroll back to the cell below the active cell on arrow down key', async ({
-  page,
-  tmpPath
-}) => {
-  await page.notebook.openByPath(`${tmpPath}/${fileName}`);
+const scrollOnKeyPressCases: {
+  key: string;
+  showCell: 'first' | 'second' | 'neither';
+  times: number;
+  enterEditor: boolean;
+}[] = [
+  {
+    // When pressing arrow down the second cell should become selected.
+    key: 'ArrowDown',
+    showCell: 'second',
+    times: 1,
+    enterEditor: false
+  },
+  {
+    // Pressing Alt does not cause any input nor cursor movement so it should
+    // not cause any scrolling. This test in particular tests against an easy
+    // mistake of force-focusing the active editor which is out of view which
+    // can cause partial scrolling in the direction of the editor. Because the
+    // scrolling is only partial multiple presses are needed.
+    key: 'Alt',
+    showCell: 'neither',
+    times: 10,
+    enterEditor: true
+  },
+  {
+    // Because the cursor starts at the beginning of the first cell, a single
+    // press of PageDown should just move the cursor to the end, which should
+    // reveal the editor by scrolling to the first cell.
+    key: 'PageDown',
+    showCell: 'first',
+    times: 1,
+    enterEditor: true
+  },
+  {
+    // Pressing `PageDown` multiple times should scroll the notebook in a way
+    // which hides both cells (even though the first press would reveal them).
+    key: 'PageDown',
+    showCell: 'neither',
+    times: 10,
+    enterEditor: true
+  }
+];
+test.describe('Scrolling on keyboard interaction when active editor is above the viewport', () => {
+  for (const testCase of scrollOnKeyPressCases) {
+    test(`Show ${testCase.showCell} cell on pressing ${testCase.key} ${testCase.times} times`, async ({
+      page,
+      tmpPath
+    }) => {
+      await page.notebook.openByPath(`${tmpPath}/${fileName}`);
 
-  // Activate the first cell.
-  await page.notebook.selectCells(0);
-  const h = await page.notebook.getNotebookInPanel();
-  const firstCell = await h!.waitForSelector(
-    '.jp-Cell[data-windowed-list-index="0"]'
-  );
-  const secondCell = await h!.waitForSelector(
-    '.jp-Cell[data-windowed-list-index="1"]'
-  );
+      // Activate the first cell.
+      await page.notebook.selectCells(0);
+      const h = await page.notebook.getNotebookInPanelLocator();
+      const firstCell = h!.locator('.jp-Cell[data-windowed-list-index="0"]');
+      const secondCell = h!.locator('.jp-Cell[data-windowed-list-index="1"]');
+      await firstCell.waitFor();
+      await secondCell.waitFor();
 
-  const bbox = await h!.boundingBox();
-  await page.mouse.move(bbox!.x, bbox!.y);
-  await Promise.all([
-    firstCell.waitForElementState('hidden'),
-    secondCell.waitForElementState('hidden'),
-    page.mouse.wheel(0, 1200)
-  ]);
+      if (testCase.enterEditor) {
+        await page.notebook.enterCellEditingMode(0);
+        // Move cursor in the first cell to the beginning of the source code
+        await page.keyboard.press('Home');
+      }
 
-  // Select cell below the active cell
-  await page.keyboard.press('ArrowDown');
+      // Position the mouse in the bounding box to allow for scrolling with mouse wheel
+      const bbox = await h!.boundingBox();
+      await page.mouse.move(bbox!.x, bbox!.y);
 
-  // Expect the second cell to become visible again.
-  await secondCell.waitForElementState('visible');
+      // Scroll down to hide the first and second cell
+      await Promise.all([
+        firstCell.waitFor({ state: 'hidden' }),
+        secondCell.waitFor({ state: 'hidden' }),
+        page.mouse.wheel(0, 1200)
+      ]);
+
+      // Press the key as many times as requested
+      for (let i = 0; i < testCase.times; i++) {
+        await page.keyboard.press(testCase.key);
+        // Allow for small delay between pressing keys
+        await page.waitForTimeout(100);
+      }
+
+      if (testCase.showCell === 'neither') {
+        // For negative test case we need to add an explicit timeout to test
+        // against the possibility of the scroll happening with a small delay.
+        await page.waitForTimeout(400);
+        await expect(firstCell).toBeHidden();
+        await expect(secondCell).toBeHidden();
+      } else {
+        const cell = testCase.showCell === 'first' ? firstCell : secondCell;
+        // Expect the cell to become visible again.
+        await cell.waitFor({ state: 'visible' });
+      }
+    });
+  }
 });
 
 test('should detach a markdown code cell when scrolling out of the viewport', async ({
@@ -247,20 +308,20 @@ test('should detach a markdown code cell when scrolling out of the viewport', as
 }) => {
   await page.notebook.openByPath(`${tmpPath}/${fileName}`);
 
-  const h = await page.notebook.getNotebookInPanel();
-  const mdCellSelector = '.jp-MarkdownCell[data-windowed-list-index="2"]';
-  const mdCell = await h!.waitForSelector(mdCellSelector);
+  const h = await page.notebook.getNotebookInPanelLocator();
+  const mdCell = h!.locator('.jp-MarkdownCell[data-windowed-list-index="2"]');
+  await mdCell.waitFor();
 
   const bbox = await h!.boundingBox();
   await page.mouse.move(bbox!.x, bbox!.y);
   await Promise.all([
-    mdCell.waitForElementState('hidden'),
+    mdCell.waitFor({ state: 'hidden' }),
     page.mouse.wheel(0, 1200)
   ]);
 
   let found = true;
   try {
-    await h!.waitForSelector(mdCellSelector, { timeout: 150 });
+    await mdCell.waitFor({ timeout: 150 });
   } catch (r) {
     found = false;
   }
@@ -273,15 +334,15 @@ test('should reattach a markdown code cell when scrolling back into the viewport
 }) => {
   await page.notebook.openByPath(`${tmpPath}/${fileName}`);
 
-  const h = await page.notebook.getNotebookInPanel();
-  const mdCellSelector = '.jp-MarkdownCell[data-windowed-list-index="2"]';
-  const mdCell = await h!.waitForSelector(mdCellSelector);
+  const h = await page.notebook.getNotebookInPanelLocator();
+  const mdCell = h!.locator('.jp-MarkdownCell[data-windowed-list-index="2"]');
+  await mdCell.waitFor();
 
   const bbox = await h!.boundingBox();
   await page.mouse.move(bbox!.x, bbox!.y);
   await Promise.all([
-    mdCell.waitForElementState('hidden'),
-    h!.waitForSelector('.jp-MarkdownCell[data-windowed-list-index="6"]'),
+    mdCell.waitFor({ state: 'hidden' }),
+    h!.locator('.jp-MarkdownCell[data-windowed-list-index="6"]').waitFor(),
     page.mouse.wheel(0, 1200)
   ]);
 
@@ -289,7 +350,7 @@ test('should reattach a markdown code cell when scrolling back into the viewport
 
   await page.mouse.wheel(0, -1200);
 
-  expect(await h!.waitForSelector(mdCellSelector)).toBeDefined();
+  await expect(mdCell).toBeVisible();
 });
 
 test('should remove all cells including hidden outputs artifacts', async ({
@@ -298,12 +359,12 @@ test('should remove all cells including hidden outputs artifacts', async ({
 }) => {
   await page.notebook.openByPath(`${tmpPath}/${fileName}`);
 
-  const h = await page.notebook.getNotebookInPanel();
+  const h = await page.notebook.getNotebookInPanelLocator();
 
   const bbox = await h!.boundingBox();
   await page.mouse.move(bbox!.x, bbox!.y);
   await Promise.all([
-    h!.waitForSelector('.jp-MarkdownCell[data-windowed-list-index="6"]'),
+    h!.locator('.jp-MarkdownCell[data-windowed-list-index="6"]').waitFor(),
     page.mouse.wheel(0, 1200)
   ]);
 
@@ -314,20 +375,69 @@ test('should remove all cells including hidden outputs artifacts', async ({
   await page.keyboard.press('d');
 
   // Check that the notebook only contains one cell
-  expect(await (await h!.$('.jp-WindowedPanel-inner'))!.textContent()).toEqual(
+  expect(await h!.locator('.jp-WindowedPanel-inner')!.textContent()).toEqual(
     '[ ]:'
   );
 
   // Check there are no hidden cells
   let found = true;
   try {
-    await h!.waitForSelector('.jp-Cell', { state: 'hidden', timeout: 150 });
+    await h!.locator('.jp-Cell').waitFor({ state: 'hidden', timeout: 150 });
   } catch (r) {
     found = false;
   }
   expect(found).toEqual(false);
 });
 
+test('should display cells below on scrolling after inserting a cell on top', async ({
+  page,
+  tmpPath
+}) => {
+  // Regression test against "disappearing cells" issue:
+  // https://github.com/jupyterlab/jupyterlab/issues/16978
+  await page.notebook.openByPath(`${tmpPath}/${fileName}`);
+
+  const notebook = await page.notebook.getNotebookInPanelLocator()!;
+  const firstCell = notebook.locator('.jp-Cell[data-windowed-list-index="1"]');
+  const lastCell = notebook.locator('.jp-Cell[data-windowed-list-index="18"]');
+  await firstCell.waitFor();
+
+  const bbox = await notebook.boundingBox();
+  await page.mouse.move(bbox!.x, bbox!.y);
+
+  // Needs to be two separate mouse wheel events.
+  await page.mouse.wheel(0, 3000);
+  await page.mouse.wheel(0, 3000);
+
+  // Scroll down to reveal last cell to ensure these all items have been measured...
+  await Promise.all([
+    firstCell.waitFor({ state: 'hidden' }),
+    lastCell.waitFor()
+  ]);
+
+  await page.mouse.wheel(0, -3000);
+  await page.mouse.wheel(0, -3000);
+
+  // ...then scroll back up and select first cell.
+  await Promise.all([
+    lastCell.waitFor({ state: 'hidden' }),
+    firstCell.waitFor()
+  ]);
+  await page.notebook.selectCells(0);
+
+  // Insert cell below.
+  await page.keyboard.press('b');
+  await page.mouse.wheel(0, 3000);
+  await page.mouse.wheel(0, 3000);
+
+  // Scroll down again.
+  await Promise.all([
+    firstCell.waitFor({ state: 'hidden' }),
+    lastCell.waitFor()
+  ]);
+  await expect(lastCell).toBeVisible();
+});
+
 test('should center on next cell after rendering markdown cell and advancing', async ({
   page,
   tmpPath
@@ -450,7 +560,7 @@ test('should navigate to a search hit in a out-of-viewport cell', async ({
   await page.getByText('1/2').waitFor();
   await expect
     .soft(page.locator('.jp-Cell[data-windowed-list-index="11"]'))
-    .toHaveCount(0);
+    .not.toBeVisible();
 
   await page.getByRole('button', { name: 'Next Match (Ctrl+G)' }).click();
 
@@ -463,7 +573,7 @@ test('should navigate to a search hit in a out-of-viewport cell', async ({
   await page.getByText('1/1').waitFor();
   await expect
     .soft(page.locator('.jp-Cell[data-windowed-list-index="18"]'))
-    .toHaveCount(0);
+    .not.toBeVisible();
 
   await page
     .getByRole('button', { name: 'Previous Match (Ctrl+Shift+G)' })
